@@ -25,8 +25,6 @@ from kazoo.handlers import threading as threading_handler
 from kazoo.protocol import paths
 from oslo_utils import encodeutils
 from oslo_utils import strutils
-import six
-from six.moves import filter as compat_filter
 
 import tooz
 from tooz import coordination
@@ -97,11 +95,17 @@ class KazooDriver(coordination.CoordinationDriverCachedRunWatchers):
     ================  ===============================  ====================
     Name              Source                           Default
     ================  ===============================  ====================
-    hosts             url netloc + 'hosts' option key  localhost:2181
-    timeout           'timeout' options key            10.0 (kazoo default)
+    ca                'ca' options key                 None
+    certfile          'certfile' options key           None
     connection_retry  'connection_retry' options key   None
     command_retry     'command_retry' options key      None
+    hosts             url netloc + 'hosts' option key  localhost:2181
+    keyfile           'keyfile' options key            None
+    keyfile_password  'keyfile_password' options key   None
     randomize_hosts   'randomize_hosts' options key    True
+    timeout           'timeout' options key            10.0 (kazoo default)
+    use_ssl           'use_ssl' options key            False
+    verify_certs      'verify_certs' options key       True
     ================  ===============================  ====================
 
     .. _kazoo: http://kazoo.readthedocs.org/
@@ -444,8 +448,10 @@ class KazooDriver(coordination.CoordinationDriverCachedRunWatchers):
         args.extend(more_args)
         cleaned_args = []
         for arg in args:
-            if isinstance(arg, six.binary_type):
-                cleaned_args.append(arg.decode('ascii'))
+            if isinstance(arg, bytes):
+                cleaned_args.append(
+                  encodeutils.safe_decode(arg, incoming='ascii')
+                )
             else:
                 cleaned_args.append(arg)
         return paths.join(*cleaned_args)
@@ -467,18 +473,24 @@ class KazooDriver(coordination.CoordinationDriverCachedRunWatchers):
             auth_data = None
 
         maybe_hosts = [parsed_url.netloc] + list(options.get('hosts', []))
-        hosts = list(compat_filter(None, maybe_hosts))
+        hosts = list(filter(None, maybe_hosts))
         if not hosts:
             hosts = ['localhost:2181']
         randomize_hosts = options.get('randomize_hosts', True)
         client_kwargs = {
-            'hosts': ",".join(hosts),
-            'timeout': float(options.get('timeout', self.timeout)),
+            'auth_data': auth_data,
+            'ca': options.get('ca', None),
+            'certfile': options.get('certfile', None),
             'connection_retry': options.get('connection_retry'),
             'command_retry': options.get('command_retry'),
-            'randomize_hosts': strutils.bool_from_string(randomize_hosts),
-            'auth_data': auth_data,
             'default_acl': default_acl,
+            'hosts': ",".join(hosts),
+            'keyfile': options.get('keyfile', None),
+            'keyfile_password': options.get('keyfile_password', None),
+            'randomize_hosts': strutils.bool_from_string(randomize_hosts),
+            'timeout': float(options.get('timeout', self.timeout)),
+            'use_ssl': bool(options.get('use_ssl', False)),
+            'verify_certs': bool(options.get('verify_certs', True)),
         }
         handler_kind = options.get('handler')
         if handler_kind:
@@ -502,7 +514,7 @@ class KazooDriver(coordination.CoordinationDriverCachedRunWatchers):
         if group_id not in self._leader_locks:
             self._leader_locks[group_id] = self._coord.Lock(
                 self._path_group(group_id) + "/leader",
-                self._member_id.decode('ascii'))
+                encodeutils.safe_decode(self._member_id, incoming='ascii'))
         return self._leader_locks[group_id]
 
     def get_leader(self, group_id):
@@ -516,11 +528,11 @@ class KazooDriver(coordination.CoordinationDriverCachedRunWatchers):
     def get_lock(self, name):
         z_lock = self._coord.Lock(
             self._paths_join(b"/", self._namespace, b"locks", name),
-            self._member_id.decode('ascii'))
+            encodeutils.safe_decode(self._member_id, incoming='ascii'))
         return ZooKeeperLock(name, z_lock)
 
     def run_elect_coordinator(self):
-        for group_id in six.iterkeys(self._hooks_elected_leader):
+        for group_id in self._hooks_elected_leader.keys():
             leader_lock = self._get_group_leader_lock(group_id)
             if leader_lock.is_acquired:
                 # Previously acquired/still leader, leave it be...
